﻿#ifndef ARBRE_HPP
#define ARBRE_HPP

#include <iostream>
using namespace std;


template <class T> class Arbre { 

private:     

struct Node2 {
  T info;
  Node2* segE;
  Node2* segD;
};

Node2* primer;


static Node2* copia_estruct(Node2* m) {
/* Pre: cert */
/* Post: el resultat és NULL si m és NULL; en cas contrari, el
 resultat apunta al node arrel d'una jerarquia de nodes que és una
 còpia de la jerarquia de nodes que té el node apuntat per m com a
 arrel */
Node2* n;
if (m == NULL) n = NULL;
else {
  n = new Node2;
  n->info = m->info;
  n->segE = copia_estruct(m->segE);
  n->segD = copia_estruct(m->segD);
}
return n;
}



static void esborra_estruct(Node2* m) {    
/* Pre: cert */
/* Post no fa res si m és NULL; en cas contrari, allibera espai de
 tots els nodes de la jerarquia que té el node apuntat per m com a
 arrel */
if (m != NULL) {
  esborra_estruct(m->segE);
  esborra_estruct(m->segD);
  delete m;
}
}



public:


Arbre() {
/* Pre: cert */
/* Post: el p.i. és un arbre buit */
primer = NULL;
}


Arbre(const Arbre& original) {
/* Pre: cert */
/* Post: el p.i. és una còpia d'original */
  primer = copia_estruct(original.primer);
}


~Arbre() {
// Esborra automàticament els objectes locals en sortir d'un àmbit
// de visibilitat.
esborra_estruct(primer);
}


Arbre& operator=(const Arbre& original) {
/* Pre: cert */
/* Post: El p.i. passa a ser una còpia d'original i qualsevol contingut
   anterior del p.i. ha estat esborrat (excepte si el p.i. i original
   ja eren el mateix objecte). */
if (this != &original) { 
  esborra_estruct(primer);
  primer = copia_estruct(original.primer);
}
return *this;
}


void a_buit() {
/* Pre: Cert */
/* Post: El p.i. és un arbre buit i qualsevol contingut anterior del
   p.i. ha estat esborrat. */
esborra_estruct(primer);
primer = NULL;
}        


void plantar(const T &x, Arbre &a1, Arbre &a2) {
/* Pre: El p.i. és buit, a1=A1, a2=A2, el p.i. no és el mateix objecte
   que a1 ni que a2. */
/* Post: el p.i. és un arbre amb arrel igual a x, amb fill esquerre
 igual a A1 i amb fill dret igual a A2; a1 i a2 són buits */
Node2* aux = new Node2;
aux->info = x;
aux->segE = a1.primer;

if (a1.primer != a2.primer or a2.primer == NULL) {
  aux->segD = a2.primer;
}
else aux->segD = copia_estruct(a2.primer);

primer = aux;
a1.primer = NULL;
a2.primer = NULL;
}


void fills (Arbre &fe, Arbre &fd) {
/* Pre: el p.i. no és buit i li diem A, fe i fd són buits i no són el
   mateix objecte */
/* Post: fe és el fill esquerre d'A, fd és el fill dret d'A, el
 p.i. és buit */
Node2* aux = primer;
fe.primer = aux->segE;
fd.primer = aux->segD;
primer = NULL;
delete aux;
}


T arrel() const {
/* Pre: el p.i. no és buit */
/* Post: el resultat és l'arrel del p.i. */
  return primer->info;    
}


bool es_buit() const {
/* Pre: cert */
/* Post: el resultat indica si el p.i. és un arbre buit */
return primer == NULL;
}


#include "sol_arboles_A.hh"  

  //#include "sol_arboles_B.hh"  

  //#include "a_max_min.hh"  
  
};
#endif
