﻿#ifndef ARBRENARI_HPP
#define ARBRENARI_HPP

#include <iostream>
#include <vector>
using namespace std;

template <class T> class ArbreNari { 

private:     

  struct Node {
    T info;
    vector<Node*> seg;
  };

  int N;
  Node* primer;


  /* Todo árbol n-ario no vacío tiene exactamente n hijos, vacíos o
     no. */



static Node* copia_jerarquia(Node* m) { 
/* Pre: Cert */
/* Post: El resultat és NULL si m és NULL; en cas contrari, el
   resultat apunta al node arrel d'una jerarquia de nodes que és una
   còpia de la jerarquia de nodes que té el node apuntat per m com a
   arrel. */
Node* n;
if (m == NULL) n = NULL;
else {
  n = new Node;
  n->info = m->info;

  // Es preciso dimensionar el vector n->seg
  int ari = m->seg.size();
  n->seg = vector<Node*>(ari);

  for (int i = 0; i < ari; ++i) {
    n->seg[i] = copia_jerarquia(m->seg[i]);
  }
}
return n;
}


static void esborra_jerarquia(Node* m) {  
/* Pre: Cert */
/* Post: No fa res si m és NULL; en cas contrari, allibera espai de
   tots els nodes de la jerarquia que té el node apuntat per m com a
   arrel. */
if (m != NULL) {
  int ari = m->seg.size();
  for (int i = 0; i < ari; ++i) {
    esborra_jerarquia(m->seg[i]);
  }
  delete m;
}
}



public:


ArbreNari(int n) {
/* Pre: n > 0 */
/* Post: el p.i. és un arbre buit d'aritat n */
  N = n;
  primer = NULL;
}


ArbreNari(const ArbreNari& original) {
/* Pre: n>0 */
/* Post: el p.i. és una còpia d'original */
N = original.N;
primer = copia_jerarquia(original.primer);
}


ArbreNari(const T &x, int n) {
/* Pre: n > 0 */
/* Post: el p.i. és un arbre amb arrel x i n fills buits */
  N = n;
  primer = new Node;
  primer->info = x;
  primer->seg = vector<Node*>(N,NULL);
}


~ArbreNari() {
// Destructora: Esborra automàticament els objectes locals en
// sortir d'un àmbit de visibilitat
esborra_jerarquia(primer);
}


ArbreNari& operator=(const ArbreNari& original) { 
/* Pre: Cert */
/* Post: El p.i. passa a ser una còpia d'original i qualsevol
 contingut anterior del p.i. ha estat esborrat (excepte si el p.i. i
 original ja eren el mateix objecte). */
if (this != &original) {
  esborra_jerarquia(primer);
  N = original.N;
  primer = copia_jerarquia(original.primer);
}
return *this;
}


void a_buit() {
/* Pre: cert */
/* Post: El p.i. és un arbre N-ari buit i qualsevol contingut anterior
   del p.i. ha estat escorrat. */
esborra_jerarquia(primer);
primer = NULL;
}        


void plantar(const T &x, vector<ArbreNari> &v) {
/* Pre:  el p.i. és buit; v.size() és igual a l'aritat del p.i., 
   tots els components de v tenen la mateixa aritat que el p.i. 
   i cap d'ells és el mateix objecte que el p.i. ni el mateix 
que un altre element de v no buit. */
/* Post: el p.i. té x com a arrel i els N elements originals  
   de v com a fills, v passa a contenir arbres buits */
  primer = new Node;
  primer->info= x;
  primer->seg = vector<Node*>(N);
  for (int i = 0; i < N; ++i) {
    primer->seg[i] = v[i].primer;
    v[i].primer = NULL;
  }
}


void fills(vector<ArbreNari> &v) {
/* Pre:  el p.i. no és buit i li diem A, v és un arbre buit */
/* Post: el p.i. és buit, v passa a contenir els N fills de l'arbre A  */
  v = vector<ArbreNari>(N, ArbreNari(N));
  for (int i = 0; i < N; ++i) {
    v[i].primer = primer->seg[i]; 
  } 
  delete primer;
  primer = NULL;
}


T arrel() const {
/* Pre: el p.i. no és buit */
/* Post: el resultat és l'arrel del p.i. */
return primer->info;
}


bool es_buit() const {
/* Pre: cert */
/* Post: el resultat indica si el p.i. és un arbre buit */
return primer == NULL;
}

int aritat() const {
/* Pre: cert */
/* Post: el resultat és l'aritat del p.i. */
return N;
}



// ----------- Extensions -------------------------------------

#include "sumaN_k.hh"

#include "sustitucionN.hh"

#include "buscaN.hh"

#include "arb_max_min.hh"  

};

#endif
